# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-23 10:26
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('filesfolders', '0002_auto_20180411_1758'),
    ]

    operations = [
        migrations.RenameField(
            model_name='file',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
        migrations.RenameField(
            model_name='folder',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
        migrations.RenameField(
            model_name='hyperlink',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),

    ]
