# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-26 06:06
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('projectroles', '0007_fix_remoteproject_foreign_key'),
    ]

    operations = [
        migrations.AddField(
            model_name='projectsetting',
            name='user',
            field=models.ForeignKey(blank=True, help_text='User to which the setting belongs', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='user_settings', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='projectsetting',
            name='project',
            field=models.ForeignKey(blank=True, help_text='Project to which the setting belongs', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='settings', to='projectroles.Project'),
        ),
    ]
