# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-01-28 12:10
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("bgjobs", "0004_backgroundjob_user")]

    operations = [
        migrations.AlterModelOptions(name="backgroundjob", options={"ordering": ["-date_created"]})
    ]
