# Generated by Django 3.2.14 on 2022-08-25 10:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0020_project_has_public_children'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='project',
            name='submit_status',
        ),
    ]
