# Generated by Django 3.1.7 on 2021-03-12 10:43

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('timeline', '0005_update_jsonfield'),
    ]

    operations = [
        migrations.AlterField(
            model_name='projectevent',
            name='user',
            field=models.ForeignKey(help_text='User who initiated the event (optional)', null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]
