# Generated by Django 3.2.18 on 2023-03-27 15:58

import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import projectroles.models


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0026_delete_category_settings'),
    ]

    operations = [
        migrations.AddField(
            model_name='role',
            name='project_types',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=64), default=projectroles.models.get_role_project_type_default, help_text='Allowed project types for the role', size=None),
        ),
        migrations.AlterField(
            model_name='roleassignment',
            name='project',
            field=models.ForeignKey(help_text='Project in which role is assigned', on_delete=django.db.models.deletion.CASCADE, related_name='local_roles', to='projectroles.project'),
        ),
    ]
