# Generated by Django 3.2.17 on 2023-02-10 14:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0024_project_star_app_setting'),
    ]

    operations = [
        migrations.DeleteModel(
            name='ProjectUserTag',
        ),
    ]
