# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-17 13:18
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('adminalerts', '0003_auto_20180802_1558'),
    ]

    operations = [
        migrations.RenameField(
            model_name='adminalert',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
    ]
