# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-08 08:47
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sodarcache', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='jsoncacheitem',
            name='user',
            field=models.ForeignKey(blank=True, help_text='User who updated the item (optional)', null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]
