# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-20 15:29
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0004_rename_uuid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='Project SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='projectinvite',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='ProjectInvite SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='projectsetting',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='ProjectSetting SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='projectusertag',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='ProjectUserTag SODAR UUID', unique=True),
        ),
        migrations.AlterField(
            model_name='roleassignment',
            name='sodar_uuid',
            field=models.UUIDField(default=uuid.uuid4, help_text='RoleAssignment SODAR UUID', unique=True),
        ),
    ]
