# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-19 15:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0003_populate_roles'),
    ]

    operations = [
        migrations.RenameField(
            model_name='project',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
        migrations.RenameField(
            model_name='projectinvite',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
        migrations.RenameField(
            model_name='projectsetting',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
        migrations.RenameField(
            model_name='projectusertag',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
        migrations.RenameField(
            model_name='roleassignment',
            old_name='omics_uuid',
            new_name='sodar_uuid'
        ),
    ]
