# Generated by Django 3.1.7 on 2021-04-07 15:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projectroles', '0019_project_public_guest_access'),
        ('timeline', '0006_update_user_optional'),
    ]

    operations = [
        migrations.AlterField(
            model_name='projectevent',
            name='project',
            field=models.ForeignKey(help_text='Project to which the event belongs (null for no project)', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='events', to='projectroles.project'),
        ),
    ]
