# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-02 13:58
from __future__ import unicode_literals

from django.db import migrations, models
import markupfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('adminalerts', '0002_adminalert_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='adminalert',
            name='_description_rendered',
            field=models.TextField(editable=False, null=True),
        ),
        migrations.AddField(
            model_name='adminalert',
            name='description_markup_type',
            field=models.CharField(choices=[('', '--'), ('html', 'HTML'), ('plain', 'Plain'), ('markdown', 'Markdown'), ('restructuredtext', 'Restructured Text')], default='markdown', editable=False, max_length=30),
        ),
        migrations.AlterField(
            model_name='adminalert',
            name='description',
            field=markupfield.fields.MarkupField(blank=True, help_text='Full description of alert (optional, will be shown on a separate page)', null=True, rendered_field=True),
        ),
    ]
